package de.bsvrz.vew.ereigniskal.ereigniskal;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Map;
import java.util.TimerTask;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.ereigniskal.ereigniskal.ereignis.Ereignis;
import de.bsvrz.vew.ereigniskal.ereigniskal.ereignis.ListeZustandsWechsel;
import de.bsvrz.vew.ereigniskal.ereigniskal.ereignis.SenderEreignis;
import de.bsvrz.vew.ereigniskal.ereigniskal.erinnerungsfunktion.ErinnerungsFunktionJahr;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.SystemkalenderEintrag;

public class NeuBerechnung extends TimerTask
{
  Map<String, Ereignis> evtList;

  Debug _debug;

  Long zeit;

  /**
   * Konstruktor
   * 
   * @param evtList
   *          Ereignisliste
   * @param zeit
   *          wird fr das Nchste neuberechnen bentigt
   */
  public NeuBerechnung(Map<String, Ereignis> evtList, Long zeit)
  {
    this.evtList = evtList;
    this.zeit = zeit;
    _debug = Debug.getLogger();
  }

  @Override
  public void run()
  {

    SimpleDateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    Calendar cal = new GregorianCalendar().getInstance();

    int jahr = cal.get(Calendar.YEAR);

    int jahrNeu = jahr + 1;
    int jahrEnd = jahrNeu + 1;

    Date von = null;
    Date bis = null;
    try
    {
      von = df.parse("01.01." + jahrNeu + " 00:00:00,000");

      bis = df.parse("01.01." + jahrEnd + " 00:00:00,000");
    }
    catch (ParseException e1)
    {
      // TODO Auto-generated catch block
      e1.printStackTrace();
    }
    // TODO Auto-generated method stub

    for (Ereignis e : evtList.values())
    {
//        _debug.fine("Neuberechnung von " + e.getPid() + " : " + von + " - " + bis);
//      if (e.getPid().equals("ereignis.ostersonntag"))
      {

//        _debug.fine("Neuberechnung von " + e.getPid() + " : " + von + " - " + bis);

        if (e.getErinnerungsFunktion() != null)
        {

          ListeZustandsWechsel lzw = new ListeZustandsWechsel();

          lzw.setListeZustandsWechsel(e.getListeZustandsWechselZeit().getListeZustandsWechsel());

          SystemkalenderEintrag ske = e.getSke();

          if (ske != null){
            lzw.getListeZustandsWechsel().putAll(ske.berecheneZustandsWechselVonBis(von.getTime(), bis.getTime()));
            
            // e.getErinnerungsFunktion().unload();
            e.setListeZustandsWechselZeit(lzw);
            e.setErinnerungsFunktion(von.getTime(), bis.getTime());            
          }
          
          

        }

      }
      
      long zeit = System.currentTimeMillis();
      ResultData result = e.berechneDatenEreignisZustand(zeit);

      ClientDavInterface connection = EreignisKalender.getConnection();
      
      if (connection != null){
        SenderEreignis sender = new SenderEreignis(connection, SenderRole.source());
        sender.sende(result);        
      }
      
      
    }

    cal.setTimeInMillis(zeit);
    int next = cal.get(Calendar.YEAR) + 1;
    cal.set(Calendar.YEAR, next);
    ErinnerungsFunktionJahr ef = new ErinnerungsFunktionJahr(new NeuBerechnung(evtList, cal.getTimeInMillis()));
    ef.load(cal.getTimeInMillis());
   
  }

}
