/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.ereigniskal.ereigniskal.erinnerungsfunktion;

import java.util.TimerTask;

import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.ConfigurationException;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.ereigniskal.ereigniskal.EreignisKalender;

/**
 * Die Klasse zum Versenden der Ereigniszustaende. Erweitert die Klasse TimerTask. Die run() Methode wird ausgefuehrt
 * wenn die im ReminderService eingestellte Zeit abgelaufen ist. Sie implentiert zustzlich das ClientSenderInterface
 * welches die Methoden zum Versenden der Daten bereitstellt.
 * 
 * @version $Revision: 1.2 $ / $Date: 2010/07/30 08:55:47 $ / ($Author: Pittner $)
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 */
public class SenderZeitgesteuert extends TimerTask implements ClientSenderInterface
{
  
  /**
   * Der Datensatz der versendet wird
   */
  private ResultData _resultData;

  /**
   * Die Pid des Ereignisses
   */
  private String _pid;

  /**
   * Der Debugger, stellt verschiedene Protokollierungsfunktionen zur Verfgung
   */
  private Debug _debug;
  

  /**
   * Konstruktor der Klasse, mit Zeitangabe des Zustandswechsels
   * 
   * @param e
   *          das Ereignis, welches den Zustand meldet
   * @param now
   *          Die Zeit bis die Daten versendet werden sollen
   * 
   */
  SenderZeitgesteuert(ResultData result)
  {
    _resultData = result;
    _pid = result.getObject().getPid();
    _debug = Debug.getLogger();
  }
  
  /*
   * (non-Javadoc)
   * 
   * @see java.util.TimerTask#run()
   */
  public void run()
  {

    try
    {

      EreignisKalender.getConnection().sendData(_resultData);
      _debug.fine("send : " + _resultData.getData());
    }
    catch (DataNotSubscribedException e)
    {
      // TODO Auto-generated catch block
      // e.printStackTrace();

      try
      {
        EreignisKalender.getConnection().subscribeSender(this, _resultData.getObject(), _resultData.getDataDescription(), SenderRole.source());
        _debug.fine("subscribe : " + _pid + "!");
      }
      catch (ConfigurationException e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
      catch (OneSubscriptionPerSendData e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
      try
      {
        EreignisKalender.getConnection().sendData(_resultData);
        _debug.fine("send : " + _resultData.getData());
      }
      catch (DataNotSubscribedException e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
      catch (ConfigurationException e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
      catch (SendSubscriptionNotConfirmed e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
    }
    catch (ConfigurationException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    catch (SendSubscriptionNotConfirmed e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

  }

  /**
   * Abstrakte Methode, die implementiert werden muss: bernimmt das tatschliche Senden der Daten
   */

  /*
   * (non-Javadoc)
   * 
   * @see stauma.dav.clientside.ClientSenderInterface#dataRequest(stauma.dav.configuration.interfaces.SystemObject,
   *      stauma.dav.clientside.DataDescription, byte)
   */
  public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
  {
    
  }

  /**
   * Abstrakte Methode, die implementiert werden muss: Zeigt, ob die Sendesteuerung erwnscht ist.
   */
  /*
   * (non-Javadoc)
   * 
   * @see stauma.dav.clientside.ClientSenderInterface#isRequestSupported(stauma.dav.configuration.interfaces.SystemObject,
   *      stauma.dav.clientside.DataDescription)
   */
  public boolean isRequestSupported(SystemObject so, DataDescription dd)
  {
    return true;
  }

}
