/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.ereigniskal.ereigniskal.senderanfrage;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.Comparator;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.apache.commons.collections.MultiHashMap;
import org.apache.commons.collections.MultiMap;

import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.ereigniskal.ereigniskal.EreignisKalender;
import de.bsvrz.vew.ereigniskal.ereigniskal.ereignis.Ereignis;
import de.bsvrz.vew.ereigniskal.ereigniskal.ereignis.ListeZustandsWechsel;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.LogischeVerknuepfung;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.SystemkalenderEintrag;

/**
 * Die Klasse zum Versenden der EreignisKalenderAntwort. Sie implentiert das ClientSenderInterface welches die Methoden
 * zum Versenden der Daten bereitstellt.
 * 
 * @version $Revision: 1.7 $ / $Date: 2015/06/09 12:57:57 $ / ($Author: Pittner $)
 * 
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 * 
 */
public class SenderAnfrage implements ClientSenderInterface
{

  /**
   * Das Datenmodell
   */
  private DataModel _datenmodell;

  /**
   * Die Daten EreignisKalenderAnfrage
   */
  private ResultData result;

  /**
   * Empfaengerrrolle
   */
  private ListeZustandsWechsel lzwV = new ListeZustandsWechsel();

  /**
   * Empfaengerrrolle
   */
  private ListeZustandsWechsel lzwZ = new ListeZustandsWechsel();

  /**
   * Der Debugger, stellt verschiedene Protokollierungsfunktionen zur Verfgung
   */
  private Debug _debug;

  private Boolean flag = false;

  /**
   * Konstruktor der Klasse
   * 
   * @param result
   *          Die Daten EreignisKalenderAnfrage
   */
  public SenderAnfrage(ResultData result)
  {
    this.result = result;
    _debug = Debug.getLogger();
  }

  /**
   * Das Ergebis der Anfrage wird ermittelt, aufbereitet und versendet
   * 
   * @return true, wenn das Versenden der Daten erfolgreich war
   */
  public Boolean send()
  {

    _debug.fine("Anfrage erhalten");

    _debug.fine("empf. Daten: " + result.getData());

    _datenmodell = EreignisKalender.getConnection().getDataModel();

    AttributeGroup atgSend = _datenmodell.getAttributeGroup("atg.ereignisKalenderAntwort");

    Aspect aspSend = _datenmodell.getAspect("asp.antwort");

    Data dataSend = EreignisKalender.getConnection().createData(atgSend);

    Data data = result.getData();

    String von = data.getItem("Anfangszeitpunkt").valueToString();
    String bis = data.getItem("Endzeitpunkt").valueToString();

    Date dtVon = new Date();
    Date dtBis = new Date();

    SimpleDateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

    try
    {

      Boolean flagVerkehrlich = false;
      
      dtVon = df.parse(von);
      dtBis = df.parse(bis);

      String absId = data.getItem("absenderZeichen").asTextValue().getValueText();

      dataSend.getTextValue("absenderZeichen").setText(absId);

      String typOption = result.getData().getItem("EreignisTypenOption").asTextValue().getValueText();

      Array rg = result.getData().getArray("RumlicheGltigkeit");

      String[] ra = new String[rg.getLength()];

      if (rg.getLength() > 0)
      {

        for (int i = 0; i < ra.length; i++)
        {

          ra[i] = rg.getItem(i).asTextValue().getValueText();
          _debug.fine("Raeumliche Gueltigkeit: " + ra[i]);

        }

      }

      Array tr = result.getData().getArray("EreignisTypReferenz");

      String[] sa = new String[tr.getLength()];

      if (sa.length > 0)
      {

        for (int i = 0; i < tr.getLength(); i++)
        {

          sa[i] = tr.getItem(i).asTextValue().getValueText();

        }

      }

      Map<String, Ereignis> tmpList = new HashMap<String, Ereignis>();

      if (sa.length == 0 && ra.length == 0)
      {

        tmpList.putAll(EreignisKalender.getEvtList());

      }
      else
      {

        // Vorbesetzten der Negativ- oder Gesamtliste
        if (typOption.equalsIgnoreCase("Nicht") || typOption.equalsIgnoreCase("Alle"))
          tmpList.putAll(EreignisKalender.getEvtList());

        // Alle Referenzen abarbeiten
        for (String s : sa)
        {

          if (typOption.equalsIgnoreCase("Nur"))
          {

            for (Ereignis e : EreignisKalender.getEvtList().values())
            {

              if (s.equalsIgnoreCase(e.getTypReferenz()))
              {
                tmpList.put(e.getPid(), e);
              }

            }

          }
          else if (typOption.equalsIgnoreCase("Nicht"))
          {

            for (Ereignis e : EreignisKalender.getEvtList().values())
            {

              if (s.equalsIgnoreCase(e.getTypReferenz()))
              {

                if (tmpList.remove(e.getPid()) != null)
                {
                  // _debug.config("Loeschen OK!");
                }
                else
                  _debug.fine("Fehler beim Loeschen !");
              }

            }

          }
          else if (typOption.equalsIgnoreCase("Alle"))
          {

            // _debug.config("Alle!");

          }
          else
          {

            _debug.fine("Fehler EreignisTypenOption!");
            return false;

          }

        }
      }

      int len = 0;

      // Sortierte Map mit den zu sendenden Daten wird erstellt
      // SortedMap<String, String[]> sendMap = new TreeMap<String, String[]>(new Comp());

      MultiMap sendMap = new MultiHashMap();

      // HashMap<String, String[]> sendMap = new HashMap<String, String[]>();

      // Feld mit nderungsinformationen wird erstellt
      Boolean[] bl = new Boolean[tmpList.size()];

      int loc = 0;

      // Eintragen der Ereignisse und Berechnen der Anzahl
      for (Map.Entry<String, Ereignis> meEvt : tmpList.entrySet())
      {

        Ereignis evt = meEvt.getValue();

        List lrg = evt.getRaeumlicheGueltigkeit();

        // Innerhalb der Anfrage gibt es einen Bezug zur raeuml. Gueltigk.
        if (ra.length > 0)
        {
          Boolean gefunden = false;

          if (lrg.size() == 0)
          {

            gefunden = true;

          }
          else
          {

            for (String s : ra)
            {

              if (lrg.contains(s))
              {
                gefunden = true;
              }

            }

            if (!gefunden)
            {
              continue;

            }
            else
            {

            }

          }

        }

        lzwZ = new ListeZustandsWechsel();
        lzwV = new ListeZustandsWechsel();

        if (EreignisKalender.getSkeList().containsKey(evt.getSkeReferenz()))
        {

          Calendar cal = new GregorianCalendar();          
          cal.setTime(dtVon);

          
          SystemkalenderEintrag ske = EreignisKalender.getSkeList().get(evt.getSkeReferenz());
          
          SortedMap<Long, Boolean> map = ske.berecheneZustandsWechselVonBis(dtVon.getTime(), dtBis.getTime());
          
//          if (ske.getPid().equals("ske.nw.vrz-nrw.es.sts.donnerstagvoreinemfeiertag")){
//            
//            
//            System.out.println("###" + ske.getName() + " -> " + map.size());
//            for (Map.Entry<Long, Boolean> me2 : map.entrySet())
//            {
//              Date d = new Date(me2.getKey());
//              System.out.println("   *** " + d + " : " + me2.getValue());
//            }
//            
//          }
          
          
////          if (ske instanceof LogischeVerknuepfung)
//          {
//            HashMap<String, Boolean> tmp2 = new HashMap<String, Boolean>();
//            
//            EreignisKalender.holeSkeRekursivNeu(ske.getPid(), tmp2);
//            
//            for (Map.Entry<String, Boolean> me : tmp2.entrySet())
//            {
//              if (EreignisKalender.getSkeList().containsKey(me.getKey())){
//                                
////              System.out.println("KEY: " + me.getKey());
//                
//                SystemkalenderEintrag skeTmp = EreignisKalender.getSkeList().get(me.getKey());
//                if (skeTmp != null){
//                  
//                  SortedMap<Long, Boolean> positivList = skeTmp.berecheneZustandsWechselVonBis(dtVon.getTime(), dtBis.getTime());
//                  
//                  if (ske.getPid().equals("ske.nw.vrz-nrw.es.sts.donnerstagvoreinemfeiertag"))
//                  {
//                    
//                    
//                    System.out.println("!!!" + ske.getName() + " -> " + positivList.size());
//                    for (Map.Entry<Long, Boolean> me2 : positivList.entrySet())
//                    {
//                      Date d = new Date(me2.getKey());
//                      System.out.println(me.getKey() + "   >>> " + d + " : " + me2.getValue());
//                    }
//                    
//                  }
//                  
//                  map.putAll(positivList);
//                  
////                if (!me.getValue())
////                {
////                  SortedMap<Long, Boolean> negativList = LogischeVerknuepfung.berechneNegativZustandsWechsel("" + cal.get(Calendar.YEAR),  "" + cal.get(Calendar.YEAR));
////                  if (positivList != null){
////                                         
////                    for (Long key : positivList.keySet())
////                    {
////                      Date date = new Date();
////                      date.setTime(key);
////                      negativList.remove(key);          
////                    }
////                    
////                    
////                    skeTmp.setListeZustandsWechsel(negativList);
////                    
////                  }
////                }
//                }              
//              }
//            }
//            
//          }
          
          
          lzwZ.setListeZustandsWechsel(map);
          
          
          Date d1 = df.parse("01.01." + cal.get(Calendar.YEAR) + " 00:00:00,000");
          
          cal.setTime(dtBis);
          
          Date d2 = df.parse("31.12." + cal.get(Calendar.YEAR) + " 23:59:59,999");
          
          SortedMap<Long, Boolean> tmp = ske.berecheneZustandsWechselVonBis(d1.getTime(), d2.getTime());
          
//          tmp2 = new HashMap<String, Boolean>();
//          EreignisKalender.holeSkeRekursivNeu(ske.getPid(), tmp2);
//          
//          for (Map.Entry<String, Boolean> me : tmp2.entrySet())
//          {
//            if (EreignisKalender.getSkeList().containsKey(me.getKey())){
//              SystemkalenderEintrag skeTmp = EreignisKalender.getSkeList().get(me.getKey());
//              if (skeTmp != null){
//
//                SortedMap<Long, Boolean> positivList = skeTmp.berecheneZustandsWechselVonBis(d1.getTime(), d2.getTime());
////                if (!me.getValue())
////                {
////                  SortedMap<Long, Boolean> negativList = LogischeVerknuepfung.berechneNegativZustandsWechsel("" + cal.get(Calendar.YEAR),  "" + cal.get(Calendar.YEAR));
////                  if (positivList != null){
////                                         
////                    for (Long key : positivList.keySet())
////                    {
////                      Date date = new Date();
////                      date.setTime(key);
////                      negativList.remove(key);          
////                    }
////                    skeTmp.setListeZustandsWechsel(negativList);                    
////                  }
////                }
//              }             
//            }
//          }
          
          
          
          for (int i = 0; i < evt.lenVerkehrlicheGueltigkeit; i++)
          {

            if (evt.anf[i] != null && evt.bezAnf[i] != null && evt.end[i] != null && evt.bezEnd[i] != null)
            {
              
              lzwV.setGueltigkeitVerkehr(tmp, evt.anf[i], evt.bezAnf[i], evt.end[i],
                  evt.bezEnd[i]);
              
              SortedMap<Long, Boolean> map2 = lzwV.berechneVonBis(dtVon.getTime(), dtBis.getTime());
                            
              lzwV.setListeZustandsWechsel(map2);
              
//              lzwV.setGueltigkeitVerkehr(lzwZ.getListeZustandsWechsel(), evt.anf[i], evt.bezAnf[i], evt.end[i],
//                  evt.bezEnd[i]);

            }
            else
            {

              _debug.fine("keine Verkehrliche Gltigkeit definiert");

            }

          }

        }
        else
        {

          SortedMap<Long, Boolean> list = evt.getListeZustandsWechselZeit().getListeZustandsWechsel();

          lzwZ.setListeZustandsWechsel(evt.getListeZustandsWechselZeit().getListeZustandsWechsel());

          lzwV.setListeZustandsWechsel(evt.getListeZustandsWechselVerkehr().getListeZustandsWechsel());

        }

        // Besitzt das Ereignis auch eine zeitl Aenderungs-Liste?
        if (meEvt.getValue().getListeZustandsWechselZeit().getListeZustandsWechsel() != null)
        {

          SortedMap<Long, Boolean> sm = lzwZ.berechneVonBis(dtVon.getTime(), dtBis.getTime());

          // Zustandswechsel vorhanden
          bl[loc] = true;

          // Es wurde kein Ereigniswechsel im definierten Zeitraum festgestellt
          if (sm == null)
          {

            // kein Zustandswechsel beim Ereignis
            bl[loc] = false;

            Map.Entry<Long, Boolean> sme = lzwZ.berechneLetztenZustandsWechsel(dtVon.getTime());

            if (sme != null)
            {
              // ... dann nimm den letzten der aufgetreten ist!
              if (sme.getValue())
              {
                Long l = sme.getKey();
                Date d = new Date();
                d.setTime(l);
                Boolean b = sme.getValue();

                SortedMap<Long, Boolean> tmp = new TreeMap<Long, Boolean>();
                tmp.put(l, b);

                sm = tmp;
              }
              else
                continue;

            }
            else
            {
              // ... es gibt immer noch keinen!
              // _debug.config(evt.getPid() + ": kein Ereigniswechsel vor dem Anfangszeitpunkt!");
              continue;
            }

          }

          for (Map.Entry<Long, Boolean> meAend : sm.entrySet())
          {

            Long time = meAend.getKey();

            Boolean gueltig = meAend.getValue();

            String[] sf = new String[3];

            sf[0] = meEvt.getKey();

            if (gueltig)
              sf[1] = "Ja";
            else
              sf[1] = "Nein";

            if (meEvt.getValue().getListeZustandsWechselZeit().getListeZustandsWechsel() != null)
            {

              // Ereignis e = meEvt.getValue();

              Date d = new Date();
              d.setTime(time);

              Map.Entry<Long, Boolean> mx = lzwV.berechneLetztenZustandsWechsel(time);

              if (mx != null)
              {
                Boolean verkehrlichGueltig = mx.getValue();
                if (verkehrlichGueltig){
                  sf[2] = "Ja";
                  flagVerkehrlich = true;
                }
                else
                  sf[2] = "Nein";
              }
              else
              {

                sf[2] = "Nein";

              }

            }
            else
            {

              sf[2] = "Nein";

            }

            Calendar cal = new GregorianCalendar();
            cal.setTimeInMillis(time);

            if ((cal.get(Calendar.MILLISECOND) == 999) && !meAend.getValue())
            {

              time += 1;

            }

            // Es wird ein eindeutiger Schluessel erzeugt
            String uniqueKey = time + "_" + meEvt.getValue().getPid();

            // sendMap.put(uniqueKey, sf);
            sendMap.put(time, sf);

            len++;

          }

        }
        else
        {
          // Ereignis hat keine Aenderungs-Liste
          _debug.fine(evt.getPid() + " hat keine zeitl. nderungsliste");
        }

        // Besitzt das Ereignis auch eine verkehrl. Aenderungs-Liste?
        if (meEvt.getValue().getListeZustandsWechselVerkehr().getListeZustandsWechsel() != null)
        {
          SortedMap<Long, Boolean> sm = lzwV.berechneVonBis(dtVon.getTime(), dtBis.getTime());

          // Zustandswechsel vorhanden
          bl[loc] = true;

          // Es wurde kein Ereigniswechsel im definierten Zeitraum festgestellt
          if (sm == null)
          {

            // kein Zustandswechsel beim Ereignis
            bl[loc] = false;

            Map.Entry<Long, Boolean> sme = lzwV.berechneLetztenZustandsWechsel(dtVon.getTime());

            if (sme != null)
            {
              // ... dann nimm den letzten der aufgetreten ist!
              if (sme.getValue())
              {
                Long l = sme.getKey();
                Date d = new Date();
                d.setTime(l);
                Boolean b = sme.getValue();

                SortedMap<Long, Boolean> tmp = new TreeMap<Long, Boolean>();
                tmp.put(l, b);

                sm = tmp;
              }
              else
                continue;

            }
            else
            {
              // ... es gibt immer noch keinen!
              continue;
            }

          }

          for (Map.Entry<Long, Boolean> meAend : sm.entrySet())
          {

            Long time = meAend.getKey();

            Boolean gueltig = meAend.getValue();

            String[] sf = new String[3];

            sf[0] = meEvt.getKey();

            if (gueltig){
              sf[2] = "Ja";
              flagVerkehrlich = true;
            }
            else
              sf[2] = "Nein";

            if (meEvt.getValue().getListeZustandsWechselZeit().getListeZustandsWechsel() != null)
            {

              Ereignis e = meEvt.getValue();

              Date d = new Date();
              d.setTime(time);

              Map.Entry<Long, Boolean> mx = lzwZ.berechneLetztenZustandsWechsel(time);

              if (mx != null)
              {
                Boolean zeitlichGueltig = mx.getValue();
                if (zeitlichGueltig)
                  sf[1] = "Ja";
                else
                  sf[1] = "Nein";
              }
              else
              {

                sf[1] = "Nein";

              }

            }
            else
            {

              sf[2] = "Nein";

            }

            Calendar cal = new GregorianCalendar();
            cal.setTimeInMillis(time);

            if ((cal.get(Calendar.MILLISECOND) == 999) && !gueltig)
            {

              time += 1;

            }

            // Es wird ein eindeutiger Schluessel erzeugt
            String uniqueKey = time + "_" + meEvt.getValue().getPid();

            // sendMap.put(uniqueKey, sf);
            
            boolean vorhanden = false;
            
            if (sendMap.containsKey(time)){
              Collection coll = (Collection)sendMap.get(time);
              
              for (Object object : coll)
              {
                // Zeitinfo wird aus dem Schluessel extrahiert
                String[] value = (String[])object;
                if (value[0].equals(sf[0]) && value[1].equals(sf[1]) && value[2].equals(sf[2])){
                  vorhanden = true;
                }                  
              }
              
            }
            
            
            if(!vorhanden)
            {
              sendMap.put(time, sf);            
              len++;       
            }else{
              
            }
          
          }

        }
        else
        {
          // Ereignis hat keine verkehrl. Aenderungs-Liste
          _debug.fine(evt.getPid() + " hat keine verkehrl. nderungsliste");
        }

        loc++;

      }// for (Map.Entry<String, Ereignis> meEvt : tmpList.entrySet())

      _debug.fine("length: " + len);

      // Ist ein Zustandswechsel in den Ereignissen vorhanden?
      Boolean aenderung = false;
      for (Boolean b : bl)
      {

        if (b == null)
        {

          continue;

        }

        if (b)
          aenderung = true; // Ja, mind. ein Ereignis wechselt den Zustand

      }

      if (aenderung)
        dataSend.getScaledValue("nderung").setText("Ja");
      else
        dataSend.getScaledValue("nderung").setText("Nein");

      // Die Anzahl der Zustandsaenderungen wird eingetragen
      // dataSend.getArray("Ereignis").setLength(sendMap.size());

      int cnt = 0;
      for (Object key : sendMap.keySet())
      {
        Collection coll = (Collection)sendMap.get(key);

        for (Object object : coll)
        {
          cnt++;
        }
      }

      dataSend.getArray("Ereignis").setLength(cnt);


      Object[] array = sendMap.keySet().toArray();

      Arrays.sort(array);

      int loci = 0;
      for (Object key : array)
      {

        Collection coll = (Collection)sendMap.get(key);
        
        for (Object object : coll)
        {
          // Zeitinfo wird aus dem Schluessel extrahiert
          String[] value = (String[])object;


          Long l = (Long)key;
          
          Date date = new Date();
          date.setTime(l);
          
          dataSend.getArray("Ereignis").getItem(loci).getTimeValue("Zeitpunkt").setMillis(l);
          dataSend.getArray("Ereignis").getItem(loci).getReferenceValue("EreignisReferenz").setText(value[0]);
          dataSend.getArray("Ereignis").getItem(loci).getScaledValue("zeitlichGltig").setText(value[1]);
          dataSend.getArray("Ereignis").getItem(loci).getScaledValue("verkehrlichGltig").setText(value[2]);

          
//          if(flagVerkehrlich){
//          
//            if (loci < array.length-1){
//              dataSend.getArray("Ereignis").getItem(loci).getScaledValue("verkehrlichGltig").setText("Ja");
//            }
//            else{ 
//              dataSend.getArray("Ereignis").getItem(loci).getScaledValue("verkehrlichGltig").setText(value[2]);
//            }
//             
//          }else{ 
//            dataSend.getArray("Ereignis").getItem(loci).getScaledValue("verkehrlichGltig").setText(value[2]);
//          }          

          loci++;

        }

      }

      // int loci = 0;
      // for (Map.Entry<String, String[]> entry : sendMap.entrySet())
      // {
      //         
      // String tmp = entry.getKey();
      //
      //        
      // // Zeitinfo wird aus dem Schluessel extrahiert
      // String millis = tmp.substring(0, tmp.indexOf("_"));
      //
      // Long l = Long.decode(millis);
      //        
      // dataSend.getArray("Ereignis").getItem(loci).getTimeValue("Zeitpunkt").setMillis(l);
      // dataSend.getArray("Ereignis").getItem(loci).getReferenceValue("EreignisReferenz").setText(entry.getValue()[0]);
      // dataSend.getArray("Ereignis").getItem(loci).getScaledValue("zeitlichGltig").setText(entry.getValue()[1]);
      // dataSend.getArray("Ereignis").getItem(loci).getScaledValue("verkehrlichGltig").setText(entry.getValue()[2]);
      //
      // loci++;
      //       
      //        
      // }

    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

    DataDescription ddSend = new DataDescription(atgSend, aspSend);

    SystemObject empfaenger = null;
    
    try
    {
      empfaenger = data.getReferenceValue("absenderId").getSystemObject();      
    }
    catch (Exception e2)
    {
      e2.printStackTrace();
      
      _debug.warning("Applikation konnte nicht ermittelt werden (in benutztem DAV nicht bekannt)");
    }
    
    if (empfaenger == null)
    {
      return false;
    }
    
    ResultData resultData = new ResultData(empfaenger, ddSend,
        EreignisKalender.getConnection().getTime(), dataSend);

    try
    {
      EreignisKalender.getConnection().sendData(resultData);
      _debug.fine("send : " + resultData.getData());
    }
    catch (DataNotSubscribedException e)
    {
      // TODO Auto-generated catch block
      // e.printStackTrace();
      try
      {
        EreignisKalender.getConnection().subscribeSender(this, resultData.getObject(), ddSend, SenderRole.sender());
        _debug.fine("subscribe!");
      }
      catch (OneSubscriptionPerSendData e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
      try
      {

        flag = false;

        synchronized (this)
        {
          try
          {
              this.wait(10000); // maximal 10 Sekunden auf Freigabe warten, sonst nicht senden
          }
          catch (InterruptedException e1)
          {
            e1.printStackTrace();
          }
        }

        if (flag)
        {
          _debug.fine("send : " + resultData.getData());
          EreignisKalender.getConnection().sendData(resultData);
        }
        else
        {
          _debug.warning("Senden an " + resultData.getObject() + " nicht mglich (DataDescription: " + ddSend + ")");
        }
        // _debug.config("send : " + resultData.getData());
      }
      catch (DataNotSubscribedException e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
      catch (SendSubscriptionNotConfirmed e1)
      {
        // TODO Auto-generated catch block
        e1.printStackTrace();
      }
    }
    catch (SendSubscriptionNotConfirmed e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

    return true;

  }

  /**
   * Abstrakte Methode, die implementiert werden muss: bernimmt das tatschliche Senden der Daten
   */
  public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
  {
    _debug.fine("SenderAnfrage : Objekt " + object.getPid() + " dd " + dataDescription + " state " + state);

    if (state == 0)
    {
      flag = true;
      synchronized (this)
      {
        notifyAll();
      }
    }

  }

  /**
   * Abstrakte Methode, die implementiert werden muss: Zeigt, ob die Sendesteuerung erwnscht ist.
   */
  public boolean isRequestSupported(SystemObject so, DataDescription dd)
  {
    return true;
  }

  /**
   * Kommentar
   * 
   * @version $Revision: 1.7 $ / $Date: 2015/06/09 12:57:57 $ / ($Author: Pittner $)
   * 
   * @author Dambach-Werke GmbH
   * @author Timo Pittner
   * 
   */
  static class Comp implements Comparator
  {
    public int compare(Object element1, Object element2)
    {
      String x = (String)element1;
      String y = (String)element2;

      String s1 = x.substring(0, x.indexOf("_"));

      Long l1 = new Long(s1);

      String s2 = y.substring(0, y.indexOf("_"));

      Long l2 = new Long(s2);

      if (l1 == l2)
        return 0;
      else if (l1 > l2)
        return 1;
      else
        return -1;
    }
  }

}
